'use strict';

exports.__esModule = true;
exports.default = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _class, _temp; /**
                    * @author v1ndic4te
                    * @copyright 2018
                    * @licence GPL-3.0
                    */

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ContextMenuWrapper = (_temp = _class = function (_Component) {
    _inherits(ContextMenuWrapper, _Component);

    function ContextMenuWrapper(props) {
        _classCallCheck(this, ContextMenuWrapper);

        var _this = _possibleConstructorReturn(this, _Component.call(this, props));

        _this.handleShowCommand = function (event) {
            var showIntent = event.detail;

            if (
            // Does not match our ID
            !(showIntent.internalId && showIntent.internalId === _this.internalId)
            // AND doesn't trigger us globally
            && !(_this.props.global && showIntent.eventDetails.triggerContext === _util.TriggerContext.Global)) return;

            (0, _util.setLastTriggerData)(_this.internalId, showIntent.data);
            _this.show(showIntent);
        };

        _this.handleShowRequest = function (event) {
            var detail = event.detail;
            var ourExternalId = _this.props.id;
            var requestExternalId = detail.externalId;
            var eventDetails = detail.eventDetails;

            if (
            // Does not match our ID
            !(ourExternalId && ourExternalId === requestExternalId)
            // AND doesn't trigger us globally
            && !(_this.props.global && eventDetails.triggerContext === _util.TriggerContext.Global)) return;

            // ...otherwise process this event as-if it was meant for us (but we're not sure yet, there may be other
            // handlers that have precedence over us.
            var showIntent = {
                internalId: _this.internalId,
                externalId: ourExternalId,
                eventDetails: eventDetails,
                data: detail.data
            };
            (0, _util.registerShowIntent)(showIntent);
        };

        _this.handleClick = function (event) {
            if (!_this.state.visible) return;

            var node = _this.nodeRef.current;
            var wasOutside = event.target !== node && !node.contains(event.target);

            if (wasOutside && _this.props.hideOnOutsideClick) _this.hide();else if (_this.props.hideOnSelfClick) {
                if (event.touches) setTimeout(function () {
                    return _this.hide();
                }, 200);else _this.hide();
            }
        };

        _this.show = function (showIntent) {
            var eventDetails = showIntent.eventDetails;
            var data = (0, _util.getLastTriggerData)(_this.internalId);
            var publicProps = _this.getPublicProps();

            if (_this.props.onShow) _this.props.onShow(data, publicProps);
            (0, _util.emitContextMenuEvent)({
                externalId: _this.props.id,
                eventName: _util.ContextMenuEvent.Show,
                data: data,
                publicProps: publicProps
            });
            (0, _util.hideAllContextMenus)();

            _this.setState({ visible: true });

            var _determineContextMenu = (0, _util.determineContextMenuPlacement)({
                clickX: eventDetails.x,
                clickY: eventDetails.y,
                domNode: _this.nodeRef.current
            }),
                x = _determineContextMenu.x,
                y = _determineContextMenu.y;

            _this.setState({
                visible: true,
                style: _extends({}, _this.state.style, {
                    left: x,
                    top: y
                })
            });
        };

        _this.hide = function () {
            if (_this.state.visible) {
                var data = (0, _util.getLastTriggerData)(_this.internalId);
                var publicProps = _this.getPublicProps();

                _this.setState({ visible: false });
                if (_this.props.onHide) _this.props.onHide(data, publicProps);
                (0, _util.emitContextMenuEvent)({
                    externalId: _this.props.id,
                    eventName: _util.ContextMenuEvent.Hide,
                    data: data,
                    publicProps: publicProps
                });
            }
        };

        _this.state = {
            node: null,
            visible: false,
            style: {}
        };

        // Generate internal ID and relevant intent handling function
        _this.internalId = (0, _util.generateInternalId)();

        // Reference to our own <div>
        _this.nodeRef = _react2.default.createRef();

        // Props that toggle simple event listeners
        _this.toggleProps = [
        // Property, Target, Event, Handler
        [null, window, _util.InternalEvent.HideAllContextMenus, _this.hide], [null, window, _util.InternalEvent.TryShowContextMenu, _this.handleShowRequest], ['hideOnScroll', document, 'scroll', _this.hide], ['hideOnWindowResize', window, 'resize', _this.hide]];

        // Warn users about bad props
        if (!_this.props.id && !_this.props.global) {
            (0, _util.warn)('A \'ContextMenuWrapper\' component you created is not global and does not have an ' + '\'id\' specified. As such, nothing can trigger it and it will never be displayed.');
        }
        return _this;
    }

    ContextMenuWrapper.prototype.updateGlobalRegistration = function updateGlobalRegistration(mounted) {
        var oldValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

        if (!oldValue) {
            if (mounted) window.addEventListener(_util.InternalEvent.DoShowContextMenu, this.handleShowCommand);else window.removeEventListener(_util.InternalEvent.DoShowContextMenu, this.handleShowCommand);
        }

        var isGlobal = this.props.global;
        if (oldValue !== null && isGlobal === oldValue) return;
        if (isGlobal && !mounted) (0, _util.removeGlobalContextMenu)(this.internalId);else if (isGlobal) (0, _util.registerGlobalContextMenu)(this.internalId);
    };

    ContextMenuWrapper.prototype.componentDidMount = function componentDidMount() {
        // Listeners for outside/inside clicks
        document.addEventListener('click', this.handleClick);
        document.addEventListener('touchstart', this.handleClick);

        // Setup toggleable handlers
        for (var _iterator = this.toggleProps, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
            var _ref;

            if (_isArray) {
                if (_i >= _iterator.length) break;
                _ref = _iterator[_i++];
            } else {
                _i = _iterator.next();
                if (_i.done) break;
                _ref = _i.value;
            }

            var toggleProp = _ref;
            var name = toggleProp[0],
                target = toggleProp[1],
                event = toggleProp[2],
                handler = toggleProp[3];

            if (!name || this.props[name]) target.addEventListener(event, handler);
        }

        this.updateGlobalRegistration(true);
    };

    ContextMenuWrapper.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
        // Update toggleable handlers
        for (var _iterator2 = this.toggleProps, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
            var _ref2;

            if (_isArray2) {
                if (_i2 >= _iterator2.length) break;
                _ref2 = _iterator2[_i2++];
            } else {
                _i2 = _iterator2.next();
                if (_i2.done) break;
                _ref2 = _i2.value;
            }

            var toggleProp = _ref2;
            var name = toggleProp[0],
                target = toggleProp[1],
                event = toggleProp[2],
                handler = toggleProp[3];

            if (name && this.props[name] !== prevProps[name]) {
                if (this.props[name]) target.addEventListener(event, handler);else target.removeEventListener(event, handler);
            }
        }

        this.updateGlobalRegistration(true, this.props.global);
    };

    ContextMenuWrapper.prototype.componentWillUnmount = function componentWillUnmount() {
        // Listeners for outside/inside clicks
        document.removeEventListener('click', this.handleClick);
        document.removeEventListener('touchstart', this.handleClick);

        // Remove toggleable handlers
        for (var _iterator3 = this.toggleProps, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _iterator3[Symbol.iterator]();;) {
            var _ref3;

            if (_isArray3) {
                if (_i3 >= _iterator3.length) break;
                _ref3 = _iterator3[_i3++];
            } else {
                _i3 = _iterator3.next();
                if (_i3.done) break;
                _ref3 = _i3.value;
            }

            var toggleProp = _ref3;
            var name = toggleProp[0],
                target = toggleProp[1],
                event = toggleProp[2],
                handler = toggleProp[3];

            if (!name || this.props[name]) target.removeEventListener(event, handler);
        }

        this.updateGlobalRegistration(false);
    };

    /**
     * @returns {object}
     */
    ContextMenuWrapper.prototype.getPublicProps = function getPublicProps() {
        var publicPropNames = Object.keys(ContextMenuWrapper.propTypes);
        var publicProps = {};
        for (var _iterator4 = publicPropNames, _isArray4 = Array.isArray(_iterator4), _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _iterator4[Symbol.iterator]();;) {
            var _ref4;

            if (_isArray4) {
                if (_i4 >= _iterator4.length) break;
                _ref4 = _iterator4[_i4++];
            } else {
                _i4 = _iterator4.next();
                if (_i4.done) break;
                _ref4 = _i4.value;
            }

            var propName = _ref4;

            publicProps[propName] = this.props[propName];
        }
        return publicProps;
    };

    ContextMenuWrapper.prototype.render = function render() {
        var style = _extends({
            zIndex: '999',
            position: 'fixed'
        }, this.state.style, {
            display: this.state.visible ? 'block' : 'none'
        });

        return _react2.default.createElement(
            'div',
            { style: style, ref: this.nodeRef },
            this.props.children
        );
    };

    return ContextMenuWrapper;
}(_react.Component), _class.defaultProps = {
    id: null,
    onShow: null,
    onHide: null,

    hideOnSelfClick: true,
    hideOnOutsideClick: true,

    // Props that toggle simple event listeners
    global: false,
    hideOnScroll: true,
    hideOnWindowResize: true
}, _temp);
exports.default = ContextMenuWrapper;
ContextMenuWrapper.propTypes = process.env.NODE_ENV !== "production" ? {
    id: _propTypes2.default.string,
    onShow: _propTypes2.default.func,
    onHide: _propTypes2.default.func,

    hideOnSelfClick: _propTypes2.default.bool,
    hideOnOutsideClick: _propTypes2.default.bool,

    // Props that toggle simple event listeners
    global: _propTypes2.default.bool,
    hideOnScroll: _propTypes2.default.bool, // TODO: Fix scroll on internal components - right now the menu won't hide.
    hideOnWindowResize: _propTypes2.default.bool
} : {};
module.exports = exports['default'];